/*
 * comportlin.cpp
 *
 *  Created on: 01.11.2010
 *      Author: Sergey Fedorenko
 */
#include <QDebug>
#include "comportlin.h"
#include "device.h"
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <signal.h>
#include <termios.h>
#include <errno.h>

ComPort::ComPort(Device * mydevice, QString setup_string) : IOPort(mydevice, setup_string)
{
    qDebug()<<"ComPortLin construct";

    m_trRead.SetPPort(this);
    m_trWrite.SetPPort(this);

    if(OpenPort())
    	qDebug()<<"ComPortLin: Port Open";
    else
    	qDebug()<<"ComPortLin: Port Error";

}

ComPort::~ComPort() {

}

bool ComPort::OpenPort()
{

	m_idComDevice = open(m_ConnectionSetup.toAscii().data(), O_RDWR | O_NOCTTY | O_NONBLOCK);
	if (m_idComDevice < 0)
		return false;


	// allow the process to receive SIGIO
	fcntl(m_idComDevice, F_SETOWN, getpid());
	// Make the file descriptor asynchronous (the manual page says only
	// O_APPEND and O_NONBLOCK, will work with F_SETFL...)
	fcntl(m_idComDevice, F_SETFL, FASYNC);


    if(!SetupConnection())
    {
        return false;
    }
    m_fConnected = true;

    if(!CheckConnection()){
        m_fConnected = FALSE;
        return FALSE;
    }

    m_trRead.start();
    m_trWrite.start();

    int reta=connect(&theComPortNotifier, SIGNAL(signalNotify()), this, SLOT(slotSentNewData()),Qt::DirectConnection);
    qDebug()<<"ComPortLin: Connect signal="<<reta;

    return true;
}
bool ComPort::ClosePort()
{
return true;
}

bool ComPort::SetupConnection()
{

	// set new port settings for canonical input processing
	struct termios newtio;
//	newtio.c_cflag = B19200 | CRTSCTS | CS8 |  CLOCAL | CREAD;
	newtio.c_cflag = B19200 | CS8 |  CLOCAL | CREAD;
	newtio.c_iflag = IGNPAR;
	newtio.c_oflag = 0;
	newtio.c_lflag = 0;       //ICANON;
	newtio.c_cc[VMIN]=1;
	newtio.c_cc[VTIME]=0;

    if(tcsetattr(m_idComDevice,TCSANOW,&newtio)<0)
        return false;

    tcflush(m_idComDevice, TCIFLUSH);

    return TRUE;
}

bool ComPort::CheckConnection()
{
    return true;
}




void ComPort::CReadThreadLin::run()
{
    unsigned char buff[COMPORTLIN_READBUFLEN];
    ssize_t readbytes;

    qDebug()<<"Read Thread Start!!!";

    m_get_new_data_flag=false;

    while(m_ptrComPort->m_fConnected) {

    	if(m_get_new_data_flag){
    		m_get_new_data_flag=false;

    		//прочитать байты из порта в буфер программы
        	do {
        		readbytes=read(m_ptrComPort->m_idComDevice, buff, COMPORTLIN_READBUFLEN);

        		if(readbytes>0) {
					m_ptrComPort->m_pParentDev->NewData(buff, (int)readbytes);
				}
			} while(readbytes>0);
    	}
		else { // если новых данных нет
			usleep(10);
    	}
    } // while connected
}



void ComPort::CWriteThreadLin::run()
{
    qDebug()<<"Write Thread Start!!!";
    unsigned char write_buff[COMPORTLIN_WRITEBUFLEN];
    int len;
    ssize_t writebytes;

    while(1)
    {
    	len=COMPORTLIN_WRITEBUFLEN; // на всякий случай передаём размер буфера в функцию
    	                            //(обработка переполнения должна быть уже внутри функции MakeData
    	                            // возвращается в этой де переменной (len) уже количество фактически
        if( m_ptrComPort->m_pParentDev->MakeData(write_buff, &len) ) {

        	writebytes=write(m_ptrComPort->m_idComDevice, write_buff, len);

			qDebug()<<"DATA were writed... (total "<<writebytes<<" bytes)";
        }
        else {
//        	qDebug()<<"No data, sleep";
        	usleep(100);
        }
    }
}

ComPortLinNotifier::ComPortLinNotifier() {

	//Глобальный обработчик сигнала SIGIO. Без обработчика программа сразу завершается.
	struct sigaction saio;
	saio.sa_handler = signal_handler_IO;
	saio.sa_flags = 0;
	saio.sa_restorer = NULL;
	sigaction(SIGIO,&saio,NULL);

}

void signal_handler_IO(int /*numsignal*/ ) {

//    qDebug()<<"---SIGIO---"<<numsignal;
    theComPortNotifier.NotifyHandler();
}

ComPortLinNotifier theComPortNotifier;
