#include <QDebug>
#include "comportwin.h"
#include "device.h"


ComPort::ComPort(Device * parent, QString _setup) : IOPort(parent, _setup)
{
    qDebug()<<"ComPort construct";

    osWrite.Offset = 0;
    osWrite.OffsetHigh = 0;
    osRead.Offset = 0;
    osRead.OffsetHigh = 0;
    //добавление эвентов
    osRead.hEvent = CreateEvent(NULL, // no security
                              TRUE,	 // explicit reset req
                              FALSE,	// initial event reset
                              NULL); // no name
    if (osRead.hEvent == NULL)	return;
    osWrite.hEvent = CreateEvent(NULL,	 // no security
                                 TRUE,	 // explicit reset req
                                 FALSE,	// initial event reset
                                 NULL); // no name
    if (NULL == osWrite.hEvent)
    {
        CloseHandle(osRead.hEvent);
        return;
    }
    //открываем порт
    if(OpenPort() == false)
    {
        CloseHandle(osRead.hEvent);
        CloseHandle(osWrite.hEvent);
        return;
    }
    qDebug()<<"ComPort construct2";
    tRead = new CReadThread(idComDevice, m_pParentDev,osRead);
    tRead->start();
    tWrite = new CWriteThread(idComDevice, m_pParentDev,osWrite);
    tWrite->start();

}
bool ComPort::OpenPort()
{
    if((idComDevice = CreateFileA(m_ConnectionSetup.toAscii().data(), GENERIC_READ | GENERIC_WRITE,
                                  0,                     // exclusive access
                                  NULL,                  // no security attrs
                                  OPEN_EXISTING,
                                  FILE_ATTRIBUTE_NORMAL |
                                  FILE_FLAG_OVERLAPPED,  // overlapped I/O
                                  NULL)) == INVALID_HANDLE_VALUE)
        return false;
    // get any early notifications
    SetCommMask(idComDevice, EV_RXCHAR);
    ClearComErrors("SetCommMask");
    // setup device buffers
    SetupComm(idComDevice, 65536, 65536);
    ClearComErrors("SetCommMask");
    // purge any information in the buffer
    PurgeComm(idComDevice, PURGE_TXABORT | PURGE_RXABORT | PURGE_TXCLEAR | PURGE_RXCLEAR);
    ClearComErrors("SetCommMask");
    // set up for overlapped I/O
    COMMTIMEOUTS CommTimeOuts;
    CommTimeOuts.ReadIntervalTimeout = 0xFFFFFFFF;
    CommTimeOuts.ReadTotalTimeoutMultiplier = 0;
    CommTimeOuts.ReadTotalTimeoutConstant = 1000;
    // CBR_9600 is approximately 1byte/ms. For our purposes, allow
    // double the expected time per character for a fudge factor.
    CommTimeOuts.WriteTotalTimeoutMultiplier = 2;
    CommTimeOuts.WriteTotalTimeoutConstant = 0;
    SetCommTimeouts( idComDevice, &CommTimeOuts);
    if(!SetupConnection())
    {
        CloseHandle(idComDevice);
        return false;
    }
    fConnected = true;
 //   Security();
 //   WriteADCState();
//Надо будет разобраться с  проверкой
//Пока что в ней жеско забито true...
    if(!CheckConnection()){
        CloseHandle(idComDevice);
        fConnected = FALSE;
        return FALSE;
    }
    return true;
}

bool ComPort::SetupConnection()
{
    DCB	dcb;
    dcb.DCBlength = sizeof(DCB);
//    if(!GetCommState( idComDev, &dcb))
//    {
//        DWORD aret=GetLastError();
//        return FALSE;
//    }

    dcb.BaudRate = CBR_19200;
    dcb.ByteSize = 8;
    dcb.Parity = NOPARITY;
    dcb.StopBits = ONESTOPBIT;

    // setup hardware flow control
    dcb.fOutxDsrFlow = 0;
    dcb.fDtrControl = DTR_CONTROL_ENABLE;
    dcb.fOutxCtsFlow = 0;
    dcb.fRtsControl = RTS_CONTROL_DISABLE;
    dcb.fDsrSensitivity = FALSE;
    // setup software flow control
    dcb.fTXContinueOnXoff = TRUE;
    dcb.fInX = dcb.fOutX = 0;
    dcb.XonChar = ASCII_XON;
    dcb.XoffChar = ASCII_XOFF;
    dcb.XonLim = 100;
    dcb.XoffLim = 100;
    // other various settings
    dcb.fBinary = TRUE;
    dcb.fParity = FALSE;
    dcb.Parity = NOPARITY;
    dcb.fNull = FALSE;
    dcb.fErrorChar = FALSE;
    dcb.fAbortOnError =TRUE;

    int setcounter=0;
    while(!SetCommState( idComDevice, &dcb))
    { // почему-то скорость перезадаётся не с первого раза, не разобрался очему :(
        ClearComErrors("SetCommState1");
        Sleep(10);
        setcounter++;
        if (setcounter>10) { // пробуем это несколько раз, на самом деле работает со второго раза
            return false;
        }
    }
    ClearComErrors("SetCommState2");
  //  qDebug()<<"SetCommState:"<<GetLastError()<<endl;
    return TRUE;
}

bool ComPort::CheckConnection()
{
    if (!SetCommMask(idComDevice, EV_RXCHAR))
        return false;
    if(!PurgeComm(idComDevice, PURGE_TXABORT | PURGE_RXABORT |  PURGE_TXCLEAR | PURGE_RXCLEAR))
    { DWORD deb=GetLastError(); deb= deb + 1 + 1 - 2; }
    DWORD dwEvtMask = 0;
    ResetEvent( osRead.hEvent);
    //	EscapeCommFunction(idComDevice, SETDTR);
    WaitCommEvent( idComDevice, &dwEvtMask, &osRead);
    DWORD res = WaitForSingleObject(osRead.hEvent, 5000);
    if ((dwEvtMask & EV_RXCHAR) != EV_RXCHAR)
    { int a = 1; a = a + 1 + 1 - 2; }
    //	EscapeCommFunction(idComDevice, CLRDTR);
    PurgeComm(idComDevice, PURGE_TXABORT | PURGE_RXABORT |  PURGE_TXCLEAR | PURGE_RXCLEAR);
    ResetEvent( osRead.hEvent);
    //	if(!SetCommMask(idComDevice, 0)); // сбрасываем флаги реакции на все события
    //		DWORD err=GetLastError();

//    return true;
    return res != WAIT_TIMEOUT;
}

void ComPort::ClearComErrors(QString str)
{
    DWORD dwErrorFlags;
    COMSTAT ComStat;
    int i = GetLastError();
    if(i != 0)
    {
        qDebug()<<str<<":"<<i<<endl;
        ClearCommError(idComDevice, &dwErrorFlags, &ComStat);
    }
}

CReadThread::CReadThread(HANDLE ComPort, Device * parent,OVERLAPPED overRead)
{
    ParentDev = parent;
    idComDevice = ComPort;//хендл устройства
    osRead = overRead;
}

void CReadThread::ClearComErrors(QString str)
{
    DWORD dwErrorFlags;
    COMSTAT ComStat;
    int i = GetLastError();
    if(i != 0)
    {
        qDebug()<<str<<":"<<i<<endl;
        ClearCommError(idComDevice, &dwErrorFlags, &ComStat);
    }
}

void CReadThread::run()
{
    COMSTAT comstat;    //структура текущего состояния порта, в данной программе используется
                        //для определения количества принятых в порт байтов
    qDebug()<<"Read Thread Start!!!";
    DWORD btr, temp, mask, signal;  //переменная temp используется в качестве заглушки
    unsigned char buff[COMPORTWIN_READBUFLEN];
    //установить маску на срабатывание по событию приёма байта в порт
    int setcounter=0;
    while(!SetCommMask(idComDevice, EV_RXCHAR))
    {
        ClearComErrors("SetCommMask1");
        Sleep(100);
        setcounter++;
        if (setcounter>10)
            break;
    }
    ClearComErrors("SetCommMask2");
    qDebug()<<"!!!:"<<GetLastError();
    while(1)//пока поток не будет прерван, выполняем цикл
    {
        //ожидать события приёма байта (это и есть перекрываемая операция)
        WaitCommEvent(idComDevice, &mask, &osRead);
        //приостановить поток до прихода байта
        signal = WaitForSingleObject(osRead.hEvent, INFINITE);
        //если событие прихода байта произошло
        if(signal == WAIT_OBJECT_0)
        {
            //проверяем, успешно ли завершилась
            //перекрываемая операция WaitCommEvent
            if(GetOverlappedResult(idComDevice, &osRead, &temp, true))
            {
                //если произошло именно событие прихода байта
                if((mask & EV_RXCHAR)!=0)
                {
                    //нужно заполнить структуру COMSTAT
                    ClearCommError(idComDevice, &temp, &comstat);
                    //и получить из неё количество принятых байтов
                    btr = comstat.cbInQue;
                    //если действительно есть байты для чтения
                    if(btr)
                    {
                        //прочитать байты из порта в буфер программы
                        ReadFile(idComDevice, buff, btr, &temp, &osRead);
                        ParentDev->NewData(buff, (int)temp);
                    }
                }
            }
        }
    }
}
CWriteThread::CWriteThread(HANDLE ComPort, Device * parent,OVERLAPPED overWrite)
{
    ParentDev = parent;
    idComDevice = ComPort;//хендл устройства
    osWrite = overWrite;
}

void CWriteThread::ClearComErrors(QString str)
{
    DWORD dwErrorFlags;
    COMSTAT ComStat;
    int i = GetLastError();
    if(i != 0)
    {
        qDebug()<<str<<":"<<i<<endl;
        ClearCommError(idComDevice, &dwErrorFlags, &ComStat);
    }
}

void CWriteThread::run()
{
    qDebug()<<"Write Thread Start!!!";
    unsigned char write_buff[COMPORTWIN_WRITEBUFLEN];
    int count = 0;
    bool fl = false;
    unsigned long temp=0, signal;
    while(1)
    {
        temp = 0;
        if( ParentDev->MakeData(write_buff, &count) )
        {
            ClearComErrors("WriteFile1");
            bool fWriteStat = WriteFile(idComDevice, write_buff, count,&temp, &osWrite);
            signal = WaitForSingleObject(osWrite.hEvent, INFINITE);	//приостановить поток, пока не завершится
                                                                            //перекрываемая операция WriteFile
             //если операция завершилась успешно, установить соответствующий флажок
             if((signal == WAIT_OBJECT_0) && (GetOverlappedResult(idComDevice, &osWrite, &temp, true))) fl = true;
             else fl = false;
            qDebug()<<"WriteFile2("<<fWriteStat<<"); GetLastError:"<<GetLastError()<<"; temp:"<<temp<<"; fl = "<<fl;
            ClearComErrors("WriteFile3");
        }
        else Sleep(1);
    }
}
